"""
    This file is part of the Microstepper project
    https://www.ohwr.org/project/microstepper
    
    This simple script computes microstepper output frequency
    given DDS frequency tuning words.
    
    released under GPLv3 or later license.
    Anders Wallin, 2019-2020
"""
import bigfloat
import numpy

def ftw_dds(f_hz):
    """ return AD9912 1GHz SYSCLK FTW
    """
    return numpy.round(f_hz*ftw_length/dds_clk,0)

def hz_dds(ftw):
    """ return f_out [Hz]
    """
    return dds_clk*ftw/ftw_length

def y_out(ftw1, ftw2, N_div=4.0, M_div=1024.0):
    """ fractional frequency of microstepper
        F_out / F_REF
    """
    with bigfloat.precision(200):
        F1term = bigfloat.BigFloat.exact(ftw1 / 4.0)
        F2term = bigfloat.BigFloat.exact(ftw2/ (N_div*M_div ))
        Fsum = F1term+F2term
        y = ftw_length* bigfloat.pow( 10.0*(Fsum), -1.0)-1.0
        return y

dds_clk=1.0e9
ftw_length = pow(2,48)
DDS2_nom = 400e6 # nominal frequency of DDS
N_div = 80.0    # first divider 2/4/8
M_div = 1024.0 # second divider 2**10 = 1024
IF = DDS2_nom/(N_div*M_div) # 97656.25 Hz
print("IF = %.3f Hz" % IF)
# nominally we get zero output frequency with these 
# tuning-wrods:
F1 = ftw_dds(400e6-4.0*IF)    # DDS1
IF1 = (100e6-hz_dds(F1)/4.0)
F2 = ftw_dds(N_div*M_div*IF1)       # DDS2
#IF2 = hz_dds(F2)/(N_div*M_div)
#F1 = ftw_dds(400e6-4.0*IF2)    # DDS1

f1_hz = hz_dds(F1)
print("f1_hz = %.12f Hz" % f1_hz)
print("IF1 = %.12f Hz" % (100e6-f1_hz/4.0))
f2_hz = hz_dds(F2)
print("f2_hz = %.12f Hz" % f2_hz)
print("IF2 = %.12f Hz" % (f2_hz/(N_div*M_div)))
print("IF1/IF2 - 1 = %.3g" % ( (100e6-f1_hz/4.0)/(f2_hz/(N_div*M_div)) - 1.0 ) )

print( 40*"-" )
print( "uStep output fractional frequency y" )
print( "FTW2 = %d (constant)" % F2 )
print( "FTW1\t\tdFTW1\ty" )
ys =[]
for delta in [-5,-4,-3,-2,-1,0,1,2,3,4,5]:
    delta=2*delta
    F1x = F1+delta
    y1 = y_out(F1x,F2, N_div = N_div) 
    ys.append(y1)
    print( "%d\t%d\t%.3g" % ( F1x, delta, y1) )
print( "y change due to 2 steps in FTW1: %.2g" % (min(numpy.diff(ys))) )
print( 40*"-" )
print( "uStep output fractional frequency y" )
print( "FTW1 = %d (constant)" % F1 )
print( "FTW2\t\tdFTW2\ty" )
ys =[]
for delta in [-5,-4,-3,-2,-1,0,1,2,3,4,5]:
    delta=2*delta
    F2x = F2+delta
    y1 = y_out(F1,F2x, N_div = N_div) 
    ys.append(y1)
    print( "%d\t%d\t%.3g" % ( F2x, delta, y1) )

print( "y change due to 2 steps in FTW2: %.3g" % (min(numpy.diff(ys))) )
print( 40*"-" )

